//************************************************************
// Quser Class Definition
//*-----------------------------------------------------------
//* Written By:  Jay J Falconer
//*              Bitwise Software International, Inc.
//*              Phoenix, AZ 85028
//*              www.ShoppingQ.com
//************************************************************
#include "ISAM.h"
#include "vsam.h"

#ifndef Quser_Class
#define Quser_Class

class Quser 
{
public:

//------------------------------
// PUBLIC functions
//------------------------------
	Quser::Quser  (void);		// declare constructor
	Quser::~Quser (void);		// declare destructor

	bool Quser::open  (void);
	bool Quser::close (void);

	bool Quser::add_rec    (string primary_key, bool rec_must_not_exist = true);
	bool Quser::update_rec (string primary_key, bool rec_must_exist    = false);
	bool Quser::delete_rec (string primary_key);

	bool Quser::get_rec    (string primary_key, bool load_data = true);
	bool Quser::find_rec   (string index_str,   bool load_data = true);

	long Quser::get_num_recs   (void);

	bool Quser::get_field_attributes (string field_name);	// gets all field attribs - load in ATTRIB structure

	bool Quser::clear_rec	  (bool load_data = true);
	bool Quser::clear_fields   (void);

	unsigned int Quser::get_num_fields (void);

	bool Quser::set_index (string new_index_name);

	bool Quser::set_BOF (void);
	bool Quser::set_EOF (void);

	bool		 Quser::error		 (void);
	unsigned int Quser::get_error_num (void);
	string		 Quser::get_error_msg (void);

	bool Quser::get_next_rec (bool load_data = true);
	bool Quser::get_prev_rec (bool load_data = true);

	bool Quser::get_field_name_list (string &list);
	bool Quser::get_field_num_list  (string &list);

	string Quser::encrypt (string text, string cypher_key);
	string Quser::decrypt (string hex_text, string cypher_key);

//------------------------------
// PUBLIC variables
//------------------------------
	// Quser record fields - loaded/accessed by calling program
	// All adding and updating of this file class must happen through these
	// public class global variables
	string rec_num;

	// Load single field alt/secondary indexes (MUST BE STRINGS)
	string email_addr;
	string user_name;
	string last_access;
	string web_site_addr;

	// load normal data fields
	string password;
	string co_name;
	string co_type;
	string addr;
	string city;
	string state;
	string zip;
	string country;
	string phone;
	string web_host_name;
	string web_server_type;
	string store_addr;
	string store_version;
	string store_user_id;
	string store_password;
	
	string night_phone;

//------------------------------------------------------------------
// PUBLIC FieldAttrib Section
//------------------------------------------------------------------
	struct attrib_rec
	{
		bool   OK;			// attrib loaded OK T/F

		string Fname;		// String Name of the field
		string Fnumber;		// Internally assigned VSAM field number
		string Fclass;		// Class
		string Ftype;		// Type: [%,&,!,#,@,$,C] 
		string Findex;		// is index - True, False
		string Fwidth;		// field width in characters
		string Fjust;		// Justify: [L,C,R]
		string Fdelete;		// Field Logically Deleted from DataSet True, False
	};

	attrib_rec ATTRIBUTES;
	

private:

//------------------------------
// PRIVATE functions
//------------------------------
	// make COPY CONSTRUCTOR "PRIVATE" to force passing this OBJ by reference and not by value
	Quser::Quser  (const Quser & CopiedObj);

	// make ASSIGNMENT OPERATOR "PRIVATE" to stop assignment of this object to another 
	void operator= (const Quser & CopiedObj);

	bool Quser::define_fields (void);

	bool Quser::get_all_fields (bool load_all_fields = true);
	bool Quser::set_all_fields (string primary_key);

//------------------------------
// PRIVATE variables/objects
//------------------------------
	ISAM DB;						// embedded private ISAM object (UNINITIALIZED)

};


#endif	// end Quser_Class
