
// vsam.h

# ifndef VSAM
# define VSAM

# ifdef LINUX
# else // not LINUX
#   ifndef RC_INVOKED
#     pragma pack(4)
#   endif // RC_INVOKED
# endif // LINUX

typedef struct tagVSTATS
  {
    LONG   nrecords;
    LONG   grp_size;
    LONG   gps_used;
    LONG   gps_unused;
    WORD   max_key_len;
    WORD   num_fields;
    LONG   nPrimRecs;
    LONG   reserved1;
    LONG   reserved2;
    LONG   reserved3;

  } VSTATS;

typedef VSTATS FAR * LPVSTATS;

# ifdef LINUX
# else // not LINUX
#   ifndef RC_INVOKED
#     pragma pack()
#   endif // RC_INVOKED
# endif // LINUX

// handle check stuff

// vsam return codes

# define VIS_OK                       0
# define VIS_NOT_FOUND                1
# define VIS_UPDATE_VIOLATION         2
# define VIS_ACCESS_DENIED            3
# define VIS_BAD_DATASET_NUMBER       4
# define VIS_BAD_HANDLE               4   // alias
# define VIS_BAD_FORMAT               5
# define VIS_INVALID_KEY              6
# define VIS_BAD_PARAMETER_VALUE      7
# define VIS_BAD_FILE                 8
# define VIS_ALREADY_EXISTS           9
# define VIS_NO_ROOM                  10
# define VIS_DISK_FULL                11
# define VIS_OUT_OF_MEMORY            12
# define VIS_DOS_ERROR                13
# define VIS_DISK_ERROR               14
# define VIS_DATA_VALIDITY_CHECK      15
# define VIS_INVALID_SECONDARY_KEY    16
# define VIS_SEQUENCE_ERROR           17
# define VIS_OUT_OF_FILE_HANDLES      18
# define VIS_BUSY                     19
# define VIS_INVALID_PASSWORD         20
# define VIS_FUNCTION_NOT_AVAILABLE   21
# define VIS_INTERUPTED               22
# define VIS_BAD_PASSWORD             23

# define VIS_NETWORK_NOT_READY        30
# define VIS_HOST_NOT_AVAILABLE       31
# define VIS_SOCKET_ERROR             32
# define VIS_CONNECT_ERROR            33
# define VIS_SOCKET_TIMEOUT           34
# define VIS_CONNECTION_REFUSED       35
# define VIS_MAX_CONNECTIONS_EXCEDED  36

# ifdef DO_SQL
# define VIS_SQL_INVALID_COMMAND      40
# endif // DO_SQL

// vsam open modes

# define READ_ONLY                    0
# define READ_ONLY_EXCLUSIVE          0
# define READ_WRITE                   1
# define READ_WRITE_EXCLUSIVE         1
# define READ_ONLY_SHARE              2
# define READ_ONLY_SHARED             2
# define READ_WRITE_SHARE             3
# define READ_WRITE_SHARED            3

// read options

# define XLOOKUP                      1
# define XNEXT                        2
# define XPREVIOUS                    4
# define XCURRENT                     8

# define XANY                         16
# define XEQ                          32
# define XBEGINS                      64

# define XNOT                         128
# define XLT                          256
# define XGT                          512
# define XLE                          1024
# define XGE                          2048

// flags for VmxGet

# define XGET_DATA                    4096
# define XNO_DATA                     8192
# define XNATIVE                      16384
# define XNOSAFE                     -32768

// flags for VmxPut

# define XADD_ONLY                    1
# define XREPLACE_ONLY                2
# define XADD_OR_REPLACE              4
# define XBATCH                       8

# define ADD_OR_REPLACE               0     // flag for write to allow add or replace
# define ADD_ONLY                     1     // flag for write to allow only adds
# define REPLACE_ONLY                 2     // flag for write to allow only replaces
# define BATCH                        8

// report format options

# define RPT_FMT_CSVHDR               1
# define RPT_FMT_CSVDATA              2
# define RPT_FMT_HRPTHDR              3
# define RPT_FMT_HRPTSEP              4
# define RPT_FMT_HRPTDATA             5
# define RPT_FMT_VRPTREC              6
# define RPT_FMT_HTMLTBLHEAD          7
# define RPT_FMT_HTMLTBLBODY          8
# define RPT_FMT_FROM_CSV_HDR         9
# define RPT_FMT_FROM_CSV_SEP         10
# define RPT_FMT_FROM_CSV_DATA        11

// enumerate field attribute options

# define FLDS_ACTIVE                  0
# define FLDS_DELETED                 1
# define FLDS_ALL                     2

// vsam function prototypes

# ifndef _CONSOLE
# ifdef __cplusplus
extern "C" {
# endif // __cplusplus
# endif // _CONSOLE

// temp !!!
int FAR PASCAL rec_decode(
    LPSTR     key,
    LPSTR     lpRecord,
    int       lRecord,
    LPSTR     buf,
    int       lbuf);

# ifndef GSTR_LOADED
# define GSTR_LOADED
typedef struct tagGSTR
  {
    WORD   size;       // length of data
    char   data[1];    // data               // <---- [0] or [1] ???

  } GSTR;

typedef GSTR FAR * LPGSTR;
typedef HANDLE HGSTR;
typedef HGSTR FAR * LPHGSTR;

// gstr function prototypes

LPGSTR FAR PASCAL StrToGstr(LPSTR lpstr, WORD len);
WORD   FAR PASCAL GstrToStr(LPSTR lpbuf, WORD lbuf, LPGSTR hgstr);
WORD   FAR PASCAL LenGstr(LPGSTR hgstr);
void   FAR PASCAL FreeGstr(LPGSTR hgstr);
# endif // GSTR_LOADED

// database functions

short FAR PASCAL VsamCreate(                   // create a database (n/c)
                     LPSTR      lpFilespec,       // database file name
                     LONG       lGroupSize,       // group size
                     LPSTR      lpDbKey);         // database encryption key
                     
short FAR PASCAL VsamKill(                     // kill (delete) a database (n/c)
                     LPSTR      lpFilespec);      // database file name

short FAR PASCAL VsamOpen(                     // open a database (n/c)
                     LPSTR      lpFilespec,       // database file name
                     WORD       wOmode,           // open mode, read only, read write, shared,
                     LPHANDLE   lpDatasetNumber,  // returned, handle of open database
                     LPSTR      lpPassword,       // password of the vsam library
                     LPSTR      lpDbKey);         // database encryption key

short FAR PASCAL VsamClose(                    // close an open database (n/c)
                     HANDLE     DatasetNumber);   // handle of open database

short FAR PASCAL VsamFlush(                    // flush modified buffers to database file (n/c)
                     HANDLE     DatasetNumber);   // handle of open database

// record input / output functions

short FAR PASCAL VsamPut(                      // write a record into database (n/c)
                     HANDLE     DatasetNumber,    // handle of open database 
                     LPSTR      lpKey,            // primary key of record
                     LPGSTR     lpRec,            // record to put
                     WORD       wProtmode);       // protect mode, only new, only replace, any

short FAR PASCAL VsamGet(                      // read a record from database (n/c)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpSecIndex,       // secondary index to use, (name or number if %n)
                     WORD       wOption,          // get option, next, previous, lookup, ...
                     LPSTR      SrchKey,          // searh key if option is lookup
                     LPSTR      SecKey,           // returned, secondary key if secindex not 0
                     LPSTR      PriKey,           // returned, primary key
                     LPGSTR    *lpRec);           // returned, pointer to record
                         
short FAR PASCAL VsamDelete(                   // delete a record from database (n/c)
                     HANDLE     DatasetNumber,    // handle of open database 
                     LPSTR      lpKey);           // primary key of record to delete

short FAR PASCAL VsamBOF(                      // move record pointer to bof in an index (n/c)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpSecIndex);      // secondary index to use, (name or number if %n)

short FAR PASCAL VsamEOF(                      // move record pointer to eof in an index (n/c)
                     HANDLE     DatasetNumber,    // handle of open database 
                     LPSTR      lpSecIndex);      // secondary index to use, (name or number if %n)

// record pointer fast move functions

short FAR PASCAL VsamMovePtr(                  // move record pointer in an index (n/c)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpSecIndex,       // secondary index to use, 0 is primary
                     LONG       lRelativeCount,   // count of records to move
                     WORD       wOption);         // option, toward bof, toward eof

short FAR PASCAL VsamCancel(                   // cancel a VsamMovePtr operation (n/c)
                     HANDLE     DatasetNumber);   // handle of open database

// dictionary functions

short FAR PASCAL VsamWriteDict(                // write a record to dict of database (n/c)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpPname,          // key of record
                     LPSTR      lpPstring);       // text of record

short FAR PASCAL VsamReadDict(                 // read a record from dict of database (n/c)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpPname,          // search key if option is lookup
                     LPSTR      lpPstring,        // returned, key of record
                     LPWORD     lpPstringSize,    // size of buffer pointed to by lpPstring
                     WORD       wOption);         // returned, text of record

short FAR PASCAL VsamDeleteDict(               // delete a record from dict of database (n/c)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpKey);           // key of record to delete

short FAR PASCAL VsamSetDictBof(               // move dictionary record pointer to bof (n/c)
                     HANDLE     DatasetNumber);   // handle of open database

short FAR PASCAL VsamSetDictEof(               // move dictionary record pointer to eof (n/c)
                     HANDLE     DatasetNumber);   // handle of open database

// informational functions

short FAR PASCAL VsamInfo(                     // retrieve database sataus information (n/c)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPVSTATS   lpVstats);        // pointer to structure to hold information

LPSTR FAR PASCAL VsamReturnCode(               // translate Vsam return code to text (n/c)
                     short      wVerrno);         // return code to translate

short FAR PASCAL VsamCb(                       // show status of open vsam file
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpBuf,            // buffer to recieve formated status
                     WORD       lBuf);            // size of buffer

// record lock/unlock functions

short FAR PASCAL VsamLock(                     // lock a text key in database (co-op) (n/c)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpKey);           // key to lock
                          
short FAR PASCAL VsamUnlock(                   // unlock a text key in database (co-op) (n/c)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpKey);           // key to unlock
                     
// field store/fetch functions
                                          
short FAR PASCAL VsamStoreField(               // store field data in a record (new)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPGSTR    *lpRecord,         // pointer to record pointer
                     LPSTR      lpField,          // field to store into (name or number if %n)
                     WORD       wElement,         // NEW array element to store
                     LPSTR      lpFldType,        // data type of field (needed?)
                     void      *lpFieldData,      // pointer to field data
                     WORD       lFieldData);      // length of field data (if a string)

short FAR PASCAL VsamSetFieldToNull(           // set a field to a null value
                     HANDLE     DatasetNumber,    // handle of open database
                     LPGSTR     lpRecord,         // pointer to record
                     LPSTR      lpField,          // field to set to null (name or number if %n)
                     WORD       wElement);        // NEW array element to set

short FAR PASCAL VsamFetchField(               // fetch field data from a record (new)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPGSTR     lpRec,            // pointer to record
                     LPSTR      lpField,          // field to fetch from (name or number if %n)
                     WORD       wElement,         // NEW array element to fetch
                     LPSTR      FldType,          // data type of field (needed?)
                     void      *lpFieldData,      // pointer to buffer for fetched data
                     WORD      *lplFieldData);    // length of buffer (only if a string)

// field definition functions

short FAR PASCAL VsamAddField(                 // add a field to a database (new)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpFieldName,      // name of field to add
                     LPSTR      lpFieldType,      // field data type
                     WORD       wDimension,       // NEW max instances of this field (dimension)
                     BOOL       bIsIndex,         // true if field is to be an index field
                     WORD       wWidth,           // field width
                     LPSTR      lpJust,           // field justification
                     LPWORD     lpFieldNumber);   // returned, number of added field

short FAR PASCAL VsamSetFieldAttribute(        // set an attribute for a field (new)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpField,          // field to access (name or number if %n)
                     LPSTR      lpAttribueName,   // name of field attribue to store
                     LPSTR      lpData);          // buffer containing attribuet data (0 term)
                                                  // NOTE: setting to null deletes the attribute

short FAR PASCAL VsamGetFieldAttribute(        // get an attribute for a field (new)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpField,          // field to access (name or number if %n)
                     LPSTR      lpAttribueName,   // name of field attribue to retrieve
                     LPSTR      lpData,           // buffer to recieve attribute data
                     LPWORD     lpwDataSize);     // size of buffer to hold field attribue

short FAR PASCAL VsamEnumAttribValues(         // get value an attribute for each field
                     HANDLE     DatasetNumber,    // handle of open database
                     WORD       wOption,          // retrieval option (0=active, 1=deleted, 2=all)
                     LPSTR      lpAttribName,     // name of field attribue to retrieve
                     LPSTR      lpAttribValues,   // a buffer to hold attribute values (',' delimited)
                     LPWORD     lpwBufSize,       // size of AttributeValues buffer
                     LPWORD     lpwNumAttribs);   // returned, actual number of field attributes

short FAR PASCAL VsamEnumFieldAttrib(          // enumerate attributes for a given field (new)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpField,          // field name/number to search
                     LPSTR      lpBuf,            // pointer to a buffer to hold field names (comma delimited)
                     LPWORD     lpwBufSize,       // size of buffer
                     LPWORD     lpwNumAttribs);   // returned, actual number of attribute names returned
                     
short FAR PASCAL VsamDeleteField(              // delete a field from database (new)
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpFieldName);     // name/number of field to delete

// extra functions

short FAR PASCAL VsamFreeRec(                  // free a record returned by VsamGet
                     LPGSTR     lpGstr);          // handle of record

short FAR PASCAL VsamVal(                      // validate integrity of a database
                     LPSTR      Filespec,         // name of database
                     LPSTR      Ebuf,             // returned, error message buffer
                     short      sFoptions,        // options
                     LPDWORD    lpFCount,         // returned, count of groups processed
                     LPSTR      EncryptKey);      // database encryption key

int   FAR PASCAL VsamMakeMap(                  // remake map file
                     LPSTR      Filespec,         // name of database
                     LPDWORD    lpFGroup,         // returned, count of groups processed
                     LPSTR      EncryptKey);      // database encryption key

short FAR PASCAL VsamRebuild(                  // rebuild database
                     LPSTR     Filespec,          // name of database
                     LPSTR     Ebuf,              // returned, error message buffer
                     short     Foptions,          // options
                     LPDWORD   Phase,             // returned, phase number
                     LPDWORD   FCount,            // returned, count of groups processed
                     LPSTR     Lkey,              // license key
                     LPSTR     EncryptKey);       // database encryption key

short FAR PASCAL VsamReport(                   // format records for reporting
                     HANDLE     DatasetNumber,    // handle of open database
                     LPSTR      lpKey,            // record key
                     LPGSTR     lpRec,            // returned, pointer to record
                     LPSTR      lpFieldList,      // list of fields to display
                     WORD       wOptions,         // output options
                     LPSTR      lpBuf,            // returned, output buffer
                     WORD       wlBuf,            // length of buffer
                     LPSTR      lpExtra);         // extra input string
 
short FAR PASCAL VsamFetchFieldAsString(       // fetch any field into a string
                     HANDLE     DatasetNumber,    // handle of open database
                     LPGSTR     lpRec,            // pointer to record
                     LPSTR      lpField,          // field to fetch from (name or number if %n)
                     WORD       wElement,         // NEW array element to store
                     void      *lpFieldData,      // pointer to string buffer for fetched data
                     WORD      *lplFieldData);    // length of string buffer
                                          
short FAR PASCAL VsamStoreFieldFromString(     // store field data from a string
                     HANDLE     DatasetNumber,    // handle of open database
                     LPGSTR    *lpRecord,         // pointer to record pointer
                     LPSTR      lpField,          // field to store into (name or number if %n)
                     WORD       wElement,         // NEW array element to store
                     void      *lpFieldData,      // pointer to field data in a string
                     WORD       lFieldData);      // length of field data (actual string only)

short FAR PASCAL VsamEncrypt(                  // encrypt a string
                     LPSTR      lpData,           // string to encrypt
                      DWORD      lData,            // length of string
                      LPSTR      eKey,              // zero terminated Encryption key
                      int        Decrypt,          // option, 0 = encrypt, 1 = decrypt
                     LPSTR      lpResult,         // result string
                     DWORD      lResult);         // length of result
                     
short FAR PASCAL VsamSearch(                   // search a database for patterns
                    LPSTR       DbName,           // database name
                    LPSTR       oFname,           // name of output file
                    LPSTR       SearchList,       // search command
                    SHORT       Foptions,         // search options
                    long       *Hits,             // returned, count of hits
                    long       *FCount,           // returned, count of groups searched
                    LPSTR       EncryptKey);      // encryption key if database encrypted

short FAR PASCAL VsamExportDdf(
                    LPSTR      DbName,            // database name 
                    LPSTR      lpExpBuf,          // buffer to recieve data defintion string 
                    DWORD      lExpBuf,           // length of export buffer 
                    LPSTR      lpDbKey);          // if DbKey is NULL - Dataset is not encrypted

short FAR PASCAL VsamImportDdf(
                    LPSTR      DbName,            // database name
                    LPSTR      lpImpBuf,          // buffer containing new data definition string
                    LPSTR      EncryptKey);       // encryption key if database encrypted

# ifndef _CONSOLE
# ifdef __cplusplus
}
# endif // __cplusplus
# endif // _CONSOLE

# endif // VSAM

// end vsam.h
